@@ -44,8 +44,16 @@ use Glib::Object::Subclass Gtk2::Window::,
     ),
   ];
 
-our $VERSION = '1.8.6';
-my $EMPTY = q{};
+our $VERSION = '1.8.8';
+my $EMPTY    = q{};
+my $HEXREGEX = qr{^(.*)           # start of message
+                  \b0x[[:xdigit:]]+\b # hex (e.g. address)
+                  (.*)$           # rest of message
+                 }xsm;
+my $INTREGEX = qr{^(.*)           # start of message
+                  \b[[:digit:]]+\b # integer
+                  (.*)$           # rest of message
+                 }xsm;
 my ($tooltips);
 
 sub INIT_INSTANCE {
@@ -261,18 +269,17 @@ sub dump_or_stringify {
     );
 }
 
-# Unpaper sometimes throws warning messages including a memory address.
-# As the address is very rarely the same, although the message itself is,
-# filter out the address from the message
+# External tools sometimes throws warning messages including a number,
+# e.g. hex address. As the number is very rarely the same, although the message
+# itself is, filter out the number from the message
 
 sub filter_message {
     my ($message) = @_;
-    my $regex = qr{^(.*)           # start of message
-                  0x[[:xdigit:]]+ # hex address
-                  (.*)$           # rest of message
-                 }xsm;
-    while ( $message =~ /$regex/xsmo ) {
-        $message =~ s/$regex/$1%%x$2/xsmo;
+    while ( $message =~ /$HEXREGEX/xsmo ) {
+        $message =~ s/$HEXREGEX/$1%%x$2/xsmo;
+    }
+    while ( $message =~ /$INTREGEX/xsmo ) {
+        $message =~ s/$INTREGEX/$1%%d$2/xsmo;
     }
     return $message;
 }
