@@ -6,6 +6,7 @@ no if $] >= 5.018, warnings => 'experimental::smartmatch';
 use Glib qw(TRUE FALSE);   # To get TRUE and FALSE
 use Image::Sane ':all';    # To get SANE_NAME_PAGE_WIDTH & SANE_NAME_PAGE_HEIGHT
 use Data::Dumper;
+use Data::UUID;
 use Storable qw(dclone);
 use feature 'switch';
 use Gscan2pdf::Dialog;
@@ -16,11 +17,11 @@ use Readonly;
 Readonly my $BORDER_WIDTH => 6;
 
 my (
-    $MAX_PAGES,         $MAX_INCREMENT,    $MAX_RELOADS,
-    $DOUBLE_INCREMENT,  $CANVAS_SIZE,      $CANVAS_BORDER,
-    $CANVAS_POINT_SIZE, $CANVAS_MIN_WIDTH, $NO_INDEX,
-    $EMPTY,
+    $MAX_PAGES,        $MAX_INCREMENT, $DOUBLE_INCREMENT,
+    $CANVAS_SIZE,      $CANVAS_BORDER, $CANVAS_POINT_SIZE,
+    $CANVAS_MIN_WIDTH, $NO_INDEX,      $EMPTY
 );
+my $uuid_object = Data::UUID->new;
 
 # need to register this with Glib before we can use it below
 BEGIN {
@@ -37,7 +38,6 @@ BEGIN {
     Readonly $CANVAS_POINT_SIZE => 10;
     Readonly $CANVAS_MIN_WIDTH  => 1;
     Readonly $NO_INDEX          => -1;
-    Readonly $MAX_RELOADS       => 5;
     $EMPTY = q{};
 }
 
@@ -71,11 +71,11 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
         param_types => ['Glib::Scalar'],    # array of options to hide
     },
     'changed-current-scan-options' => {
-        param_types => ['Glib::Scalar'],    # profile array
+        param_types => [ 'Glib::Scalar', 'Glib::String' ], # profile array, UUID
     },
     'reloaded-scan-options' => {},
     'changed-profile'       => {
-        param_types => ['Glib::Scalar'],    # name
+        param_types => ['Glib::Scalar'],                   # name
     },
     'added-profile' => {
         param_types => [ 'Glib::Scalar', 'Glib::Scalar' ], # name, profile array
@@ -253,8 +253,8 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
         'More reloads than this are considered infinite loop',    # blurb
         0,                                                        # min
         $MAX_INCREMENT,                                           # max
-        $MAX_RELOADS,                                             # default
-        [qw/readable writable/]                                   # flags
+        0,                                                        # default
+        [qw/readable/]                                            # flags
     ),
     Glib::ParamSpec->int(
         'num-reloads',                                            # name
@@ -267,7 +267,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '1.8.6';
+our $VERSION = '1.8.8';
 
 my ( $d_sane, $logger, $tooltips );
 my $tolerance             = 1;
@@ -595,7 +595,7 @@ sub _add_device_combobox {
                     if ( $_->{name} eq $device ) {
                         Gscan2pdf::Dialog::Scan::set_combobox_by_text(
                             $self->{combobd}, $_->{label} );
-                        $self->scan_options;
+                        $self->scan_options($device);
                         return;
                     }
                 }
@@ -862,6 +862,12 @@ sub _set_available_scan_options {
     else {
         $self->{framen}->set_sensitive(TRUE);
     }
+
+    # reload-recursion-limit is read-only
+    # Triangular number n + n-1 + n-2 + ... + 1 = n*(n+1)/2
+    my $n = $newval->num_options;
+    $self->{reload_recursion_limit} = $n * ( $n + 1 ) / 2;
+
     $self->signal_emit('reloaded-scan-options');
     return;
 }
@@ -977,6 +983,34 @@ sub pack_widget {
     my ( $self, $widget, $data ) = @_;
     my ( $options, $opt, $hbox, $hboxp ) = @{$data};
     if ( defined $widget ) {
+
+        # Add label for units
+        if ( $opt->{unit} != SANE_UNIT_NONE ) {
+            my $text;
+            given ( $opt->{unit} ) {
+                when (SANE_UNIT_PIXEL) {
+                    $text = $d_sane->get('pel')
+                }
+                when (SANE_UNIT_BIT) {
+                    $text = $d_sane->get('bit')
+                }
+                when (SANE_UNIT_MM) {
+                    $text = $d_sane->get('mm')
+                }
+                when (SANE_UNIT_DPI) {
+                    $text = $d_sane->get('dpi')
+                }
+                when (SANE_UNIT_PERCENT) {
+                    $text = $d_sane->get(q{%})
+                }
+                when (SANE_UNIT_MICROSECOND) {
+                    $text = $d_sane->get('us')
+                }
+            }
+            my $label = Gtk2::Label->new($text);
+            $hbox->pack_end( $label, FALSE, FALSE, 0 );
+        }
+
         $self->{option_widgets}{ $opt->{name} } = $widget;
         if ( $opt->{type} == SANE_TYPE_BUTTON or $opt->{max_values} > 1 ) {
             $hbox->pack_end( $widget, TRUE, TRUE, 0 );
@@ -992,7 +1026,6 @@ sub pack_widget {
         }
 
         $self->create_paper_widget( $options, $hboxp );
-
     }
     else {
         $logger->warn("Unknown type $opt->{type}");
@@ -1445,19 +1478,20 @@ sub edit_paper {
         __('Width')  => 'int',
         __('Height') => 'int',
         __('Left')   => 'int',
-        __('Top')    => 'int'
+        __('Top')    => 'int',
+        __('Units')  => 'text',
     );
     for ( keys %{$formats} ) {
         push @{ $slist->{data} },
           [
             $_,                $formats->{$_}{x}, $formats->{$_}{y},
-            $formats->{$_}{l}, $formats->{$_}{t}
+            $formats->{$_}{l}, $formats->{$_}{t}, 'mm',
           ];
     }
 
-    # Set everything to be editable
+    # Set everything to be editable except the units
     my @columns = $slist->get_columns;
-    for ( 0 .. $#columns ) {
+    for ( 0 .. $#columns - 1 ) {
         $slist->set_column_editable( $_, TRUE );
     }
     $slist->get_column(0)->set_sort_column_id(0);
@@ -1650,21 +1684,35 @@ sub set_profile {
 
         # If we are setting the profile, don't unset the profile name
         $self->{setting_profile} = TRUE;
+        my $uuid;
 
         # Only emit the changed-profile signal when the GUI has caught up
         my $signal;
         $signal = $self->signal_connect(
             'changed-current-scan-options' => sub {
-                $self->signal_handler_disconnect($signal);
-                $self->{setting_profile} = FALSE;
+                my ( undef, undef, $uuid_found ) = @_;
 
-                # set property before emitting signal to ensure callbacks
-                # receive correct value
-                $self->{profile} = $name;
-                $self->signal_emit( 'changed-profile', $name );
+                # there seems to be a race condition in t/0621_Dialog_Scan_CLI.t
+                # where the uuid set below is not set in time to be tested in
+                # this if.
+                if (
+                    not defined $uuid
+                    or ( defined $uuid_found
+                        and $uuid_found eq $uuid )
+                  )
+                {
+
+                    $self->signal_handler_disconnect($signal);
+                    $self->{setting_profile} = FALSE;
+
+                    # set property before emitting signal to ensure callbacks
+                    # receive correct value
+                    $self->{profile} = $name;
+                    $self->signal_emit( 'changed-profile', $name );
+                }
             }
         );
-        $self->set_current_scan_options( $self->{profiles}{$name} );
+        $uuid = $self->set_current_scan_options( $self->{profiles}{$name} );
     }
 
     # no need to wait - nothing to do
@@ -2092,11 +2140,14 @@ sub set_current_scan_options {
     # First clone the profile, as otherwise it would be self-modifying
     my $clone = dclone($profile);
 
+    # add uuid to identify later which callback has finished
+    $clone->{uuid} = $uuid_object->create_str;
+
     # Give the GUI a chance to catch up between settings,
     # in case they have to be reloaded.
     # Use the callback to trigger the next loop
     $self->_set_option_profile( $clone, $clone->each_backend_option );
-    return;
+    return $clone->{uuid};
 }
 
 sub _set_option_profile {
@@ -2183,7 +2234,8 @@ sub _set_option_profile {
         $self->{setting_current_scan_options} = FALSE;
         $self->signal_emit(
             'changed-current-scan-options',
-            $self->get('current-scan-options')
+            $self->get('current-scan-options'),
+            $profile->{uuid}
         );
     }
     return;
