@@ -32,7 +32,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '1.8.6';
+our $VERSION = '1.8.8';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -372,7 +372,6 @@ sub set_option {
             $val = $option->{constraint}{max};
         }
     }
-    $self->{current_scan_options}->add_backend_option( $option->{name}, $val );
 
     my $signal;
     my $options = $self->get('available-scan-options');
@@ -388,7 +387,13 @@ sub set_option {
             $self->signal_emit( 'changed-progress', undef, undef );
         },
         finished_callback => sub {
-            my ($data) = @_;
+            my ( $data, $status ) = @_;
+
+            if ( $status != SANE_STATUS_INVAL ) {
+                $self->{current_scan_options}
+                  ->add_backend_option( $option->{name}, $val );
+            }
+
             my $opt;
             if ($data) {
                 $self->update_options(
@@ -413,10 +418,8 @@ sub set_option {
                 # Emit the changed-current-scan-options signal
                 # unless we are actively setting it
                 if ( not $self->{setting_current_scan_options} ) {
-                    $self->signal_emit(
-                        'changed-current-scan-options',
-                        $self->get('current-scan-options')
-                    );
+                    $self->signal_emit( 'changed-current-scan-options',
+                        $self->get('current-scan-options'), $EMPTY );
                 }
             }
 
@@ -469,8 +472,17 @@ sub scan {
 
             if ( $self->get('cycle-sane-handle') ) {
                 my $current = $self->get('current-scan-options');
+                my $signal;
+                $signal = $self->signal_connect(
+                    'finished-process' => sub {
+                        my ( $widget, $data ) = @_;
+                        if ( $data eq 'find_scan_options' ) {
+                            $self->signal_handler_disconnect($signal);
+                            $self->set_current_scan_options($current);
+                        }
+                    }
+                );
                 $self->scan_options( $self->get('device') );
-                $self->set_current_scan_options($current);
             }
         },
         new_page_callback => sub {
