@@ -18,7 +18,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.8.6';
+    $VERSION = '1.8.8';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();              # eg: TAG => [ qw!name1 name2! ],
@@ -562,6 +562,9 @@ sub get_option {
         }
     }
     elsif ( defined $default->{$option} ) { return $default->{$option} }
+    elsif ( defined $hashref->{$option} ) {
+        return $hashref->{$option}{default};
+    }
     return;
 }
 
